from StringIO import StringIO

from Products.CMFCore.utils import getToolByName
from Products.ldapconfig import config
from Products.ldapconfig.config import ldapConfig
from Products.LDAPUserFolder import manage_addLDAPUserFolder
from Products.PlonePAS.Extensions.Install import activatePluginInterfaces
from Products.LDAPMultiPlugins.LDAPMultiPlugin import manage_addLDAPMultiPlugin
from Products.LDAPMultiPlugins.ActiveDirectoryMultiPlugin import manage_addActiveDirectoryMultiPlugin


from ldap import SERVER_DOWN

def install(portal):
    out = StringIO()

    print >> out, "Begin install of %s." % (config.PROJECTNAME,)
    if not ldapConfig['LDAP_HOST_MAIN']:
        print >> out, "No main ldap host found, so no LDAP MultiPlugin or User Folder added."
        print >> out, "Please update ldapconfig.py in the etc directory of your instance"
        print >> out, "if you want LDAP connectivity."
        return out.getvalue()
    if config.HAS_PLONE25:
        _addMultiPlugin(portal, out)
        _configMultiPlugin(portal, out)
    _addLdapUserFolder(portal, out)
    if ldapConfig['USE_RAM_CACHE']:
        _addRamCache(portal, out)
    print >> out, "Completed install of %s." % (config.PROJECTNAME,)

    return out.getvalue()

def _addMultiPlugin(portal, out):
    pas = getToolByName(portal, 'acl_users')
    plug_id = ldapConfig['LDAP_PLUGIN_ID']
    # Deleting our existing plugin should be fine.
    if plug_id in pas.objectIds():
        pas._delObject(plug_id)
    ls = ldapConfig['LDAP_SETTINGS']

    if ldapConfig['USE_ACTIVE_DIRECTORY']:
        print >> out, "Adding ActiveDirectory MultiPlugin."
        print >> out, "Please test if this works; try the sharing tab."
        pluginAdder = manage_addActiveDirectoryMultiPlugin
    else:
        print >> out, "Adding LDAP MultiPlugin."
        pluginAdder = manage_addLDAPMultiPlugin
    # manage_addLDAPMultiPlugin() takes at least 13 arguments...
    try:
        pluginAdder(
            self = pas,
            id = plug_id,
            title = ldapConfig['LDAP_TITLE'],
            LDAP_server = ldapConfig['LDAP_HOST_MAIN'],
            login_attr = ls['login_name_attribute'],
            uid_attr = ls['user_id_attribute'],
            users_base = ls['users_base_dn'],
            users_scope = ls['users_base_scope'],
            roles= ls['default_user_roles'],
            groups_base = ls['groups_base_dn'],
            groups_scope = ls['groups_base_dn_scope'],
            binduid = ls['manager_dn'],
            bindpwd = ls['manager_dn_password'],
            binduid_usage = ls['manager_dn_usage'],
            rdn_attr = ls['rdn_attribute'],
            local_groups = ls['group_storage'],
            use_ssl = ls['use_ssl'],
            encryption = ls['user_password_encryption'],
            read_only= ls['read_only'],
            )
    except SERVER_DOWN:
        print >> out, "WARNING: LDAP server %s is down." % (ldapConfig['LDAP_HOST_MAIN'])

def _addLdapUserFolder(portal, out):
    """Add an ldap user folder.

    For now this only configures an *existing* ldap.

    In the future this should work without PlonePAS and
    LDAPMultiPlugin as well.
    """
    print >> out, "Configure LDAP UserFolder."
    pas = getToolByName(portal, 'acl_users')
    plug_id = ldapConfig['LDAP_PLUGIN_ID']
    ls = ldapConfig['LDAP_SETTINGS']
    ldapFolder = pas[plug_id].acl_users

    # Hm, these are _private attributes. :(
    # You can change them via an edit function in LDAPUserFolder,
    # but then you need to change *all* of them.  Oh well, we should
    # probably do that anyway; it *does* give code duplication. :(
    ldapFolder._implicit_mapping = ls['group_mapping']
    ldapFolder._user_objclasses = [x.strip() for x in
                                   ls['user_object_classes'].split(',')]

    # Change server settings by deleting the old one and chucking in a
    # new one.
    ldapFolder.manage_deleteServers(position_list=[0])
    # The main ldap host does not have a timeout yet.
    ldaphosts = []
    ldaphosts.append(ldapConfig['LDAP_HOST_MAIN'])
    ldaphosts += ldapConfig['LDAP_HOSTS_EXTRA']
    for ldap_host in ldaphosts:
        ldapFolder.manage_addServer(host=ldap_host,
                                    conn_timeout=ldapConfig['LDAP_CONN_TIMEOUT'],
                                    op_timeout=ldapConfig['LDAP_OP_TIMEOUT'])

    # Configure our LDAP Schema
    ldapFolder.setSchemaConfig(ldapConfig['LDAP_SCHEMA'])

def _configMultiPlugin(portal, out):
    print >> out, "Configure LDAP MultiPlugin."
    pas = getToolByName(portal, 'acl_users')
    plug_id = ldapConfig['LDAP_PLUGIN_ID']
    assert(plug_id in pas.objectIds())

    # Found in another code section.  Not sure what it does.  Maybe
    # needed.
    #pas.updateLDAPUserFolderMapping()

    # Activate *all* interfaces:
    #activatePluginInterfaces(portal, plug_id, out)
    # Activate *specific* interfaces:
    for plugInterface in ldapConfig['ACTIVATE_PLUGINS']:
        if plug_id not in pas.plugins.listPluginIds(plugInterface):
            pas.plugins.activatePlugin(plugInterface, plug_id)
        pas.plugins.movePluginsUp(plugInterface, [plug_id])

def _addRamCache(portal, out):
    """Add a RamCache to speed up LDAP.
    You may not want this when debugging.
    """

    plug_id = ldapConfig['LDAP_PLUGIN_ID']
    pas = getToolByName(portal, 'acl_users')
    pas[plug_id].ZCacheable_setManagerId('RAMCache')
