# Configuration file for your LDAP settings.
# 
# The product first loads the config from its own ldapconfig.py.  Then
# it searches for an ldapconfig.py file in the etc/ dir of your Zope
# instance.  Any settings in there will override the previously found
# settings.  So copy the default ldapconfig.py file to
# <zope-instance>/etc/ldapconfig.py and adapt it to your liking.

# Import the plugin interfaces
from Products.ldapconfig.interfaces import *

LDAP_HOST_MAIN = None
LDAP_CONN_TIMEOUT = 5 # LDAP connection timeout in seconds
LDAP_OP_TIMEOUT = 2 # LDAP operational timeout in seconds

# Set this to true if you want an ActiveDirectory plugin instead of an
# LDAP plugin.  The ActiveDirectory plugin may not work correctly.
# Please test.
USE_ACTIVE_DIRECTORY = False

# See ldapconfig.py.example for more settings.

# Add a RAMCache.  Maybe set this to False if you are debugging.
USE_RAM_CACHE = True

#####################################################
####### Which plugins do we want to activate? #######
#####################################################

# These plugin interfaces get activated for our ldap.  Plus ldap is
# moved to the front in the list of plugin ids that are activated for
# that interface.

# Note: if you activate the IRolesPlugin the portal_role_manager still
# ends up before your LDAP plugin, but at least LDAP now comes before
# local_roles.  It might not matter.

# The base configuration here aims quite low: we want to authenticate
# against LDAP and view some lists of users.  We need to experiment a
# bit to see which plugins are really needed.  We don't want to use
# ldap groups, but some group plugins are still needed.
ACTIVATE_PLUGINS = [
    IAuthenticationPlugin,
    #ICredentialsResetPlugin,
    IGroupEnumerationPlugin,
    IGroupsPlugin,
    IPropertiesPlugin,
    IRoleEnumerationPlugin,
    IUserEnumerationPlugin,
    IRolesPlugin,
    #ICredentialsUpdatePlugin,
    #IUserAdderPlugin,
    IGroupManagement,
    IGroupIntrospection,
    #IUserManagement,
    ]

# We need to map some LDAP attributes to their Plone equivalents.  To
# get some basic functionality, Plone expects to get a fullname and an
# email out of LDAP.

LDAP_SCHEMA = {
  'cn': {
    'friendly_name': 'Full Name',
    'ldap_name': 'cn', # Common Name
    'multivalued': '',
    'public_name': 'fullname'},
  'sn': {
    'friendly_name': 'Last Name',
    'ldap_name': 'sn', # surname
    'multivalued': '',
    'public_name': ''}, # Not used in Plone actually
  'mail': {
    'friendly_name': 'E-mail',
    'ldap_name': 'mail',
    'multivalued': '',
    'public_name': 'email'},
  }
